@@ -4,6 +4,8 @@ __all__ = []
 
 import os
 import traceback
+import logging
+
 
 from a2p2.facility import Facility
 from a2p2.vlti.gui import VltiUI
@@ -48,6 +50,7 @@ On your first valid VLTI's OB, A2p2 will ask for your ESO User Portal credential
 """
 HELPTEXT += "Config files loaded from " + CONFDIR
 
+logger = logging.getLogger(__name__)
 
 class VltiFacility(Facility):
 
@@ -152,13 +155,14 @@ class VltiFacility(Facility):
             return " P2API connected with " + self.username
 
     def connectAPI(self, username, password, ob):
-        import p2api
         if username == TUTORIAL_LOGIN or username == JMMC_LOGIN:
             self.apitype = DEMO_APITYPE
         else:
             self.apitype = PRODUCTION_APITYPE
         try:
-            self.api = p2api.ApiConnection(self.apitype, username, password)
+            logger.info("Connecting to p2 api(%s).."%self.apitype)
+            self.api = P2ApiConnectionWrapper(self.apitype, username, password)
+            logger.info("Connected to p2 api")
             # TODO test that api is ok and handle error if any...
 
             self.refreshTree()
@@ -167,9 +171,9 @@ class VltiFacility(Facility):
             self.username = username
             self.ui.showTreeFrame(ob)
         except:
-            self.ui.addToLog("Can't connect to P2 (see LOG).")
+            self.ui.ShowErrorMessage("Can't connect to P2 (see LOG).")
             trace = traceback.format_exc()
-            self.ui.addToLog(trace, False)
+            self.ui.addToLog(trace, False, level=logging.ERROR)
 
     def refreshTree(self):
         runs, _ = self.api.getRuns()
@@ -193,7 +197,6 @@ class VltiFacility(Facility):
 
 # TODO Move code out of this class
 
-
 class P2Container:
     # run object stores:
     # {'pi': {'lastName': 'Accont', 'emailAddress': '52052@nodomain.net', 'firstName': 'Phase 1/2 Ttorial'},
@@ -241,3 +244,79 @@ class P2Container:
 
     def __str__(self):
         return """instrument:'%s', containerId:'%s'""" % (self.run['instrument'], self.containerId)
+
+
+# Add wrapper to wrapp p2api/requsts code and enhance
+import p2api
+from p2api import P2Error
+import json
+
+import requests
+from requests.adapters import HTTPAdapter
+from urllib3.util.retry import Retry
+#  Retry Handling for requests
+# See https://www.peterbe.com/plog/best-practice-with-retries-with-requests
+# Or https://findwork.dev/blog/advanced-usage-python-requests-timeouts-retries-hooks/
+def requests_retry_session(
+        retries=3,
+        backoff_factor=1.0,
+        status_forcelist=(500, 502, 504)):
+
+    adapter = HTTPAdapter()
+    adapter.max_retries = Retry(
+        total=retries,
+        read=retries,
+        connect=retries,
+        backoff_factor=backoff_factor,
+        status_forcelist=status_forcelist,
+        respect_retry_after_header=False
+    )
+    session = requests.Session()
+    session.mount('http://', adapter)
+    session.mount('https://', adapter)
+    return session
+
+
+requests_session = requests_retry_session()
+
+class P2ApiConnectionWrapper(p2api.ApiConnection):
+
+    def __init__(self, apitype, username, password):
+        p2api.ApiConnection.__init__(self, apitype, username, password)
+        logger.debug("init P2ApiConnectionWrapper")
+
+    def request(self, method, url, data=None, etag=None):
+        self.request_count += 1
+
+        # configure request headers
+        assert self.access_token is not None
+        headers = {
+            'Authorization': 'Bearer ' + self.access_token,
+            'Accept': 'application/json'
+        }
+        if data is not None:
+            headers['Content-Type'] = 'application/json'
+        if etag is not None:
+            headers['If-Match'] = etag
+
+        # make request
+        url = self.apiUrl + url
+        if self.debug:
+            print(method, url, data)
+        # using session makes some calls more than 2.5 X faster
+        # r = requests.request(method, url, headers=headers, data=json.dumps(data))
+        r = requests_session.request(method, url, headers=headers, data=json.dumps(data))
+        content_type = r.headers['Content-Type'].split(';')[0]
+        etag = r.headers.get('ETag', None)
+
+        # handle response
+        if 200 <= r.status_code < 300:
+            if content_type == 'application/json':
+                data = r.json()
+                return data, etag
+            return None, etag
+        elif content_type == 'application/json' and 'error' in r.json():
+            raise P2Error(r.status_code, method, url, r.json()['error'])
+        else:
+            raise P2Error(r.status_code, method, url, 'oops unknown error '+r)
+            # TODO handle 401 ? should we do something to reconnect API ...
\ No newline at end of file
